# coding=utf-8

from hiword import extract_keywords

docs = [
    """中华人民共和国中央人民政府门户网站（简称“中国政府网”）是由国务院办公厅主办，中国政府网运行中心负责运行并维护。 [1] 
中国政府网是政府面向社会的窗口，也是公众与政府互动的渠道，对于促进政务公开、推进依法行政、接受公众监督、改进行政管理、全面履行政府职能具有重要意义。
中国政府网是国务院和国务院各部门，以及各省、自治区、直辖市人民政府在国际互联网上发布政府信息和提供在线服务的综合平台。中国政府网现开通“国务院、总理、新闻、政策、互动、服务、数据、国情”等栏目，第一时间权威发布国务院重大决策部署和重要政策文件，国务院领导同志重要会议、考察、出访活动等政务信息，同时面向社会提供与政府业务相关的服务，建设基于互联网的政府与公众互动交流新渠道。
中国政府网于2005年10月1日试开通，2006年1月1日正式开通。欢迎广大公众对本网站建设积极献计献策，提出宝贵意见和建议。 [2]""",
]


def test_extract_keywords():
    for i, s in enumerate(docs):
        print('#{}: {}'.format(i + 1, s))
        print('#keywords:')
        for x, w in extract_keywords(s, with_weight=True):
            print(x, w)
